from rmidi import mutils
from rmidi.MIDI import MIDI
from rmidi import rmidi 
import numpy, random

'''
MIDI Object Testinh
'''

import os

y = MIDI.parse_midi('./midis/Believer_-_Imagine_Dragons.mid')

tr = y.track(0)

no = tr.notes()



y.compress('check')

# y.create_file('check')

e = mutils.file_hash('./midis/Believer_-_Imagine_Dragons.mid')

d = mutils.file_hash('check.mid')

print(e, ", ", d , " : ", e == d)


'''
Sequence
'''
# mu = rmidi.Muse()
# le = 7
# x = numpy.arange(12, 12 + le, 2)
# se = [1, 2, 4, 8, 16, 32]
# x = [random.choice(se) for i in range(le)]
# y = mu.sequence(le)
# m = mu.muse(x, y)
# print(m)


# m.create_file("test_add_event_ch")
# m.compress("test_add_event_ch")
# c = m.parse_midi('test-legit.mid')
# print(c)
# # c.create_file('duplicate')
# c.compress()


"""
midi_to_note testing from mutuls 
21 -> c0
60 -> c4
"""
# y = mutils.midi_to_note([56, 56, 78, 98, 34, 54, 95])

# y2 = mutils.midi_to_note(60)

# y == y2

"""
Testing the set_tempo of MIDI
"""

# x = MIDI.parse_midi('default.mid')

# x.set_tempo(56)

# x.compress('tempo_change')


"""
Sieve Erothenes 
"""

# import numpy, math
# def such(n):
#     nums = numpy.zeros(1000000, dtype = numpy.int32)
#     primes = [2]
#     i = 0
#     while primes[i] < math.sqrt(1000000) :
#         for j in range( primes[i], 1000000, primes[i]):
#             if nums[j] == 0:  nums[j] = primes[i]
#         for j in range(primes[i] + 1, 2 * primes[i]):
#             if nums[j] == 0:
#                 primes.append(j)
#                 break
#         i += 1
#     nums
#     return nums

# a = such(10)
# unique, counts = numpy.unique(a, return_counts=True)
# dct = dict(zip(unique, counts))
# n = 11
# pp = dct[n]

# print(pp)



import matplotlib.pyplot as plt
from rmidi.MIDI import MIDI
from rmidi.mutils import channel as mchannel

class Analyser:
    def __init__(self, midi_file, **kwargs):
        self.__midi = MIDI.parse_midi(midi_file)
        
    def stats(self):
        stat_freq = {'channel' : {8: 0, 9: 0, 10: 0, 11: 0, 12: 0, 13: 0, 14: 0},
             'meta': {0: 0, 1: 0, 2: 0, 3: 0, 4: 0, 5: 0, 6: 0, 7: 0, 32: 0, 33: 0, 47: 0, 81: 0, 84: 0, 88: 0, 89: 0, 127: 0}, 
             'sys': {240: 0, 247: 0}
             }
        for t in self.__midi.tracks:
            for e in t.trk_event:
                if e.is_channel_event():
                    stat_freq['channel'][mchannel(e.evt_id)] += 1
                elif e.is_meta_event():
                    stat_freq['meta'][e.meta_event_type] += 1
                elif e.is_meta_event():
                    stat_freq['sys'][e.event_id] += 1
                else : raise AttributeError('invalid MIDI event or Event Id got is not supported by this version of rmidi ')




if __name__ == "__main__":
    f = "./midis/Believer_-_Imagine_Dragons.mid"

    y = MIDI.parse_midi(f)

    t0 = y.track(0)

    n = t0.notes()
    nn = n["note_series"]
    x = [v[0] for v in nn]

    y = [v[1] for v in nn]

    plt.plot(x, y)
    plt.legend()
    plt.show()
